"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mutate = void 0;
const useAxios_1 = require("./useAxios");
const vue_query_1 = require("@tanstack/vue-query");
/**
 * A wrapper around useMutation that handles the common use case of creating a new resource.
 */
class Mutate {
    withDefaults(config) {
        return Object.assign(Object.assign({}, this.defaultConfig), (config !== null && config !== void 0 ? config : {}));
    }
    constructor(config) {
        this.defaultConfig = {
            autoInvalidate: true,
        };
        this.config = this.withDefaults(config);
    }
    create({ builderFunc, invalidate, select, }) {
        const { axios } = (0, useAxios_1.useAxios)();
        const queryClient = (0, vue_query_1.useQueryClient)();
        const triggerInvalidate = (vars) => __awaiter(this, void 0, void 0, function* () {
            if (invalidate === undefined)
                return;
            const keys = invalidate({ vars, config: this.config });
            keys === null || keys === void 0 ? void 0 : keys.forEach((key) => __awaiter(this, void 0, void 0, function* () {
                yield queryClient.invalidateQueries({
                    queryKey: key,
                });
            }));
        });
        const mutate = (0, vue_query_1.useMutation)({
            mutationFn: (vars) => __awaiter(this, void 0, void 0, function* () {
                const queryBuilder = builderFunc({ vars, config: this.config });
                const data = yield axios
                    .post(queryBuilder.path.value, queryBuilder.body.value, {
                    params: queryBuilder.params.value,
                })
                    .then((res) => (select ? select(res.data) : res.data));
                return data;
            }),
            onSuccess: (_data, vars) => __awaiter(this, void 0, void 0, function* () {
                if (this.config.autoInvalidate)
                    triggerInvalidate(vars);
            }),
        });
        return Object.assign(Object.assign({}, mutate), { triggerInvalidate });
    }
    update({ builderFunc, invalidate, }) {
        const { axios } = (0, useAxios_1.useAxios)();
        const queryClient = (0, vue_query_1.useQueryClient)();
        const triggerInvalidate = (vars) => __awaiter(this, void 0, void 0, function* () {
            if (invalidate === undefined)
                return;
            const keys = invalidate({ vars, config: this.config });
            keys === null || keys === void 0 ? void 0 : keys.forEach((key) => __awaiter(this, void 0, void 0, function* () {
                yield queryClient.invalidateQueries({
                    queryKey: key,
                });
            }));
        });
        return (0, vue_query_1.useMutation)({
            mutationFn: (vars) => __awaiter(this, void 0, void 0, function* () {
                const queryBuilder = builderFunc({ vars, config: this.config });
                const data = yield axios
                    .put(queryBuilder.path.value, queryBuilder.body.value, {
                    params: queryBuilder.params.value,
                })
                    .then((res) => res.data);
                return data;
            }),
            onSuccess: (_data, vars) => __awaiter(this, void 0, void 0, function* () {
                if (this.config.autoInvalidate)
                    triggerInvalidate(vars);
            }),
        });
    }
    delete({ builderFunc, invalidate, }) {
        const { axios } = (0, useAxios_1.useAxios)();
        const queryClient = (0, vue_query_1.useQueryClient)();
        const triggerInvalidate = (vars) => __awaiter(this, void 0, void 0, function* () {
            if (invalidate === undefined)
                return;
            const keys = invalidate({ vars, config: this.config });
            keys === null || keys === void 0 ? void 0 : keys.forEach((key) => __awaiter(this, void 0, void 0, function* () {
                yield queryClient.invalidateQueries({
                    queryKey: key,
                });
            }));
        });
        return (0, vue_query_1.useMutation)({
            mutationFn: (vars) => __awaiter(this, void 0, void 0, function* () {
                const queryBuilder = builderFunc({ vars, config: this.config });
                const data = yield axios
                    .delete(queryBuilder.path.value, queryBuilder.body.value)
                    .then((res) => res.data);
                return data;
            }),
            onSuccess: (_data, vars) => __awaiter(this, void 0, void 0, function* () {
                if (this.config.autoInvalidate)
                    triggerInvalidate(vars);
            }),
        });
    }
}
exports.Mutate = Mutate;
